#version 330

in vec2 fragTexCoord;

out vec4 color;

uniform sampler2D colorTexture;
uniform sampler2D velocityTexture;


const int MAX_VELOCITY_SAMPLES = 128;
uniform float uVelocityScale = 1.5; // current fps / target fps






void main(){

	vec2 byteVelocity = texture(velocityTexture, fragTexCoord).rg;
	
	float kVelocityInvPower = 1.0 / 3.0;
	
	
	vec2 floatVelocity = vec2(pow(byteVelocity.x, kVelocityInvPower), pow(byteVelocity.y, kVelocityInvPower));
	
		
	floatVelocity = floatVelocity * 2.0 - 1.0;
	
	vec2 velocity = floatVelocity;
	
	
	
	velocity *= uVelocityScale;
	

	vec2 texelSize = 1.0 / vec2(textureSize(colorTexture, 0));
	
	float speed = length(velocity / texelSize);
    
	int nSamples = clamp(int(speed), 1, MAX_VELOCITY_SAMPLES);
	
	vec4 result = vec4(0.0);
	
	result = texture(colorTexture, fragTexCoord);
	
	for (int i = 1; i < nSamples; ++i) {
		vec2 offset = velocity * (float(i) / float(nSamples - 1) - 0.5);
		result += texture(colorTexture, fragTexCoord + offset);
	}
	
	
	result /= float(nSamples);
	
	color = result;
	

	
}