#version 330
// world space normal mapping - vertex shader

uniform mat4 cameraMatrix;
uniform mat4 modelMatrix;

#define MAX_LIGHTS 10
uniform int numLights;
uniform struct Light {
   vec4 position;
   vec3 color;
   float attenuation;
   float ambientCE;
   float spotlightAngle;
   vec3 spotlightDirection;
} lights[MAX_LIGHTS];

in vec3 vertPosition;
in vec2 vertTexCoord;
in vec3 vertNormal;
in vec3 vertTangent;
in vec3 vertBitangent;

out vec3 fragPosition;
out vec2 fragTexCoord;
out vec3 lightDirections[MAX_LIGHTS];
out mat3 TBN;

void main() {
	// position of the vertex, in clip space: MVP * position
	gl_Position = cameraMatrix * modelMatrix * vec4(vertPosition, 1.0);

	// position of the vertex in worldspace
	fragPosition = (modelMatrix * vec4(vertPosition, 1.0)).xyz;

	// output texture coordinate
	fragTexCoord = vertTexCoord;	

	vec3 t = normalize((modelMatrix * vec4(vertTangent, 0.0)).xyz);
	vec3 b = normalize((modelMatrix * vec4(vertBitangent, 0.0)).xyz);
	vec3 n = normalize((modelMatrix * vec4(vertNormal, 0.0)).xyz);

	// matrix for transformations from tangent space to world space
	TBN = mat3(t, b, n);

	// light directions in world space
	for(int i = 0; i < numLights; ++i)
	{
		lightDirections[i] = lights[i].position.xyz - fragPosition;
	}
}